
#ifndef WADSTRUCT_H
#define WADSTRUCT_H

typedef unsigned short int16;
typedef unsigned char int8;

/* WAD structures */

typedef struct {
   char sig[4];
   int nlumps;
   int diroffset;
} WadHeader;

typedef struct {
   int offset;
   int size;
   char name[8];
} WadDirEntry;


/* Pictures and Patches */

typedef struct {
   unsigned short width,height;
   short xoffset,yoffset;
   int idx[1];
} PictHeader;

typedef struct { /* dumb-only extension */
   char sig[2]; /* == J1 */
   unsigned char log2width,log2height;
   int width,height;
   /* followed by width<<log2_height bytes of data */
   int8 data[1]; 
} AltPictData;

typedef union {
   PictHeader hdr;
   AltPictData alt;
   int8 data[1];
} PictData;

#define IS_JPATCH(p) ((p)->alt.sig[0]=='J'&&(p)->alt.sig[1]=='1')


/* Wall Textures */   

typedef struct {
   short x,y;
   unsigned short pnum;
   unsigned short stepdir; /* ??? */
   unsigned short colmap;  /* ??? */
} PatchDescData;

#define MAXPATCHDS 64

typedef struct {
   char name[8];
   int dummy1;
   short dx,dy;
   int dummy2;
   int16 npatches;
   PatchDescData patch[MAXPATCHDS];
} TextureData;
   
typedef struct {
   int num;
   char name[1];
} PNameTable;

typedef struct {
   int ntxts;
   int idx[1];
} TextureHdr;

typedef union {
   TextureHdr hdr;
   char data[1];
} TextureTable;


/*** Levels ***/

/* the thingdefs */

typedef struct {
   short x,y;
   short angle; /* this is in *degrees*, not bams */
   int16 type;
   int16 flags;
} ThingData;

#define THING_12    1
#define THING_3     2
#define THING_45    4
#define THING_DEAF  8
#define THING_MULTI 16

/* the linedefs */

#define LINE_IMPASSIBLE 0x0001
#define LINE_MIMPASSIBLE 0x0002
#define LINE_TWOSIDED 0x0004
#define LINE_UPUNPEG 0x0008
#define LINE_LOUNPEG 0x0010
#define LINE_SECRET 0x0020
#define LINE_BLKSOUND 0x0040
#define LINE_NOMAP 0x0080
#define LINE_FORCEMAP 0x0100

/* these are dumb-specific */
#define LINE_POSTER 0x0200

typedef struct {
   int16 ver2,ver1;
   int16 flags;
   int16 type;
   int16 tag;
   short side[2];
} LineData;


/* the sidedefs */

typedef struct {
   short xoffset,yoffset;
   char utexture[8];
   char ltexture[8];
   char texture[8];
   short sector;
} SideData;


/* the vertices */ 

typedef struct {
   short x,y;
} VertexData;


/* the sectordefs */

typedef struct {
   short floor,ceiling;
   char ftexture[8];
   char ctexture[8];
   int16 light;
   int16 type;
   int16 tag;
} SectorData;
   

/* the BSP data: segs, ssectors, nodes */

typedef struct {
   int16 ver1,ver2;
   int16 angle; /* this is in BAMs, not degrees */
   int16 line;
   int16 isleft; /* 1 if on 2nd side of line, otherwise 0 */
   int16 offset; /* along line */
} SegData;

typedef struct {
   int16 nsegs,seg1;
} SSectorData;

typedef struct {
   /* these coordinates refer to the Node's partition line */
   int16 x,y;
   int16 dx,dy;
   /* the boundaries of the right child-node */
   int16 rymax,rymin;
   int16 rxmin,rxmax;
   /* the boundaries of the left child-node */
   int16 lymax,lymin;
   int16 lxmin,lxmax;
   /* the children */
   int right:15;
   int rleaf:1; /* 1 if the right child is a leaf node, ie a ssector */
   int left:15;
   int lleaf:1;
} NodeData;


/* BLOCKMAP */

#define BM_BLOCKSIZE 128

typedef struct  {
   short minx,miny;
   short numx,numy;
   unsigned short idx[1];
} BlockMapHdr;

typedef union  {
   BlockMapHdr hdr;
   short data[1];
} BlockMap;
   
/* blocklist(x,y) is at data+(x-minx)/BMBS+((y-miny)/BMBS)*numx
 * "each blocklist starts with a 0 and ends with a 0xffff" */

#endif
